import pandas as pd
import matplotlib.pyplot as plt
import math

def extract_stable_data(datafile):
    datapoint = {}
    #df is a pandas data frame
    df = pd.read_csv(datafile)

    #extract phase mean and variance for stable measurements (don't ask what loc means) 
    phases = df.loc[df['EQUILIBRIUM_INDICATOR'] == 31, 'S21_PHASE']
    if phases.size == 0:
        return None
    datapoint['phase_mean']=phases.mean()
    datapoint['phase_var']=phases.var()

    magnitudes = df.loc[df['EQUILIBRIUM_INDICATOR'] == 31, 'S21_MAGNITUDE']
    datapoint['magnitude_mean'] = magnitudes.mean()
    datapoint['magnitude_var'] = magnitudes.var()

    temperatures = df.loc[df['EQUILIBRIUM_INDICATOR'] == 31, 'READBACK_TEMPERATURE']
    datapoint['temperature_mean'] = temperatures.mean()
    datapoint['temperature_var'] = temperatures.var()

    humidities = df.loc[df['EQUILIBRIUM_INDICATOR'] == 31, 'READBACK_HUMIDITY']
    datapoint['humidity_mean'] = humidities.mean()
    datapoint['humidity_var'] = humidities.var()

    return datapoint


# sweep_type is either 'temperature' or 'humidity'
def plot_sweep(temperatures, humidities, basename, sweep_type):
    x_data = []
    phases = []
    phase_vars = []
    magnitudes = []
    magnitude_vars = []
    for temp, hum in zip(temperatures, humidities):
        datafile = basename+'_'+str(temp)+'deg_'+str(hum)+'rh.csv'
        print(datafile)
        datapoint = extract_stable_data(datafile)
        if datapoint is None:
            continue

        if sweep_type == 'temperature':
            x_data.append(datapoint['temperature_mean'])
        elif sweep_type == 'humidity':
            x_data.append(datapoint['humidity_mean'])
        else:
            raise Exception('Unknown sweep_type:'+str(sweep_type))

        phases.append(datapoint['phase_mean'])
        phase_vars.append(datapoint['phase_var'])
        magnitudes.append(datapoint['magnitude_mean'])
        magnitude_vars.append(datapoint['magnitude_var'])

    magnitudes_db = [20 * math.log10(x) for x in magnitudes ]
    # approximate formula for small errors (linear approximation)
    magnitude_vars_db = [20 / math.log(10) * dx/x for x, dx in zip(magnitudes,  magnitude_vars)]

    fig, ax1 = plt.subplots()

    if sweep_type == 'temperature':
        plt.title(basename + ': Temperature sweep at ' + str(humidities[0]) + ' % r.h.')
        ax1.set_xlabel('temperature [deg C]')
    elif sweep_type == 'humidity':
        plt.title(basename + ': Humidity sweep at ' + str(temperatures[0]) + ' deg C')
        ax1.set_xlabel('relative humidity [%]')
    else:
        raise Exception('Unknown sweep_type:'+str(sweep_type))

    ax1.errorbar(x_data, phases, phase_vars, marker='+', linewidth=0)
    ax1.set_ylabel('S21 phase [deg]', color='blue')

    ax2 = ax1.twinx()
    ax2.errorbar(x_data, magnitudes_db, magnitude_vars_db, marker='x', color='red', linewidth=0)
    ax2.set_ylabel('S21 magnitude [dB]', color='red')
    
    fig.tight_layout()  # otherwise the right y-label is slightly clipped
    plt.show()


if __name__ == '__main__':
   plot_sweep(range(20, 30+1), [40]*11, 'tempsweep1', 'temperature')
